#pragma once
#include "Framework.h"
#include "Vertex.h"
#include "Matrix.h"
#include <vector>

enum class Axis { X, Y, Z };

class Rasteriser : public Framework
{
public:	
	bool Initialise();
	void Update(const Bitmap& bitmap);
	void Render(const Bitmap& bitmap);
	void ClearViewport(const Bitmap& bitmap);

	void DrawSquare(HDC hDc, const vector<Vertex> verticies);
	void DrawShape(HDC hDc, const vector<Vertex> verticies);

	Matrix GetTranslateMatrix(const float x, const float y, const float z);
	Matrix GetScaleMatrix(const float x, const float y, const float z);
	Matrix GetRotationMatrix(const Axis rotAxis, const float rotDegrees);
	Matrix GetRotationMatrixFromPoint(const Axis rotAxis, const float rotDegrees, const float x, const float y, const float z);
	
	float DegreesToRadians(const float degrees);

private:
	vector<Vertex> _initialVertexArray;
	vector<Vertex> _currentVertexArray;	
	const float _PI = (float) acos(-1);
	int _rotation;
};

